@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2020, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,10 +36,9 @@ import org.hsqldb.dbinfo.DatabaseInformation;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.FrameworkLogger;
-import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.HsqlArrayList;
 import org.hsqldb.lib.HsqlTimer;
-import org.hsqldb.lib.OrderedHashSet;
+import org.hsqldb.lib.OrderedHashMap;
 import org.hsqldb.map.ValuePool;
 import org.hsqldb.persist.HsqlDatabaseProperties;
 import org.hsqldb.persist.HsqlProperties;
@@ -61,7 +60,7 @@ import org.hsqldb.types.Collation;
  * It holds the data structures that form an HSQLDB database instance.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.1
+ * @version 2.6.0
  * @since 1.9.0
  */
 public class Database {
@@ -632,7 +631,7 @@ public class Database {
             closemode = CLOSEMODE_IMMEDIATELY;
         }
 
-        /**
+        /*
          * impact of possible error conditions in closing the log
          * for the CLOSEMODE_COMPACT mode
          */
@@ -725,7 +724,7 @@ public class Database {
 
         list.add(collation.getDatabaseCollationSQL());
 
-        HashMappedList lobTables =
+        OrderedHashMap lobTables =
             schemaManager.getTables(SqlInvariants.LOBS_SCHEMA);
 
         for (int i = 0; i < lobTables.size(); i++) {
@@ -845,8 +844,10 @@ public class Database {
                 // ignore exceptions
                 // may be InterruptedException or IOException
             } finally {
-                sysSession.commit(false);
-                sysSession.close();
+                try {
+                    sysSession.commit(false);
+                    sysSession.close();
+                } catch (Throwable t) {}
 
                 waiting = false;
             }
@@ -874,30 +875,39 @@ public class Database {
         }
     }
 
+    /**
+     * Sessions running a statement with timeout are added to the list. The first
+     * session starts the runner. Sessions are not removed until closed.<p>
+     *
+     * The runner is called at second intervals. It handles the countdown for
+     * each session currently running a statement with timeout. If timeout
+     * is reached, the runner aborts the statement.<p>
+     */
     static class TimeoutRunner implements Runnable {
 
-        private Object timerTask;
-        OrderedHashSet sessionList;
+        private Object        timerTask;
+        private HsqlArrayList sessionList;
+        int                   abortCount;
 
         public void run() {
 
             try {
-                for (int i = sessionList.size() - 1; i >= 0; i--) {
-                    Session session = (Session) sessionList.get(i);
+                synchronized (this) {
+                    for (int i = 0; i < sessionList.size(); i++) {
+                        Session session = (Session) sessionList.get(i);
 
-                    if (session.isClosed()) {
-                        synchronized (this) {
+                        if (session.isClosed()) {
                             sessionList.remove(i);
+
+                            i--;
+
+                            continue;
                         }
 
-                        continue;
-                    }
+                        boolean result = session.timeoutManager.checkTimeout();
 
-                    boolean result = session.timeoutManager.checkTimeout();
-
-                    if (result) {
-                        synchronized (this) {
-                            sessionList.remove(i);
+                        if (result) {
+                            abortCount++;
                         }
                     }
                 }
@@ -916,7 +926,6 @@ public class Database {
                 }
 
                 HsqlTimer.cancel(timerTask);
-                sessionList.clear();
 
                 timerTask   = null;
                 sessionList = null;
@@ -936,7 +945,7 @@ public class Database {
 
         private void start() {
 
-            sessionList = new OrderedHashSet();
+            sessionList = new HsqlArrayList(128);
             timerTask = DatabaseManager.getTimer().schedulePeriodicallyAfter(0,
                     1000, this, true);
         }
