@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2020, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,7 +37,7 @@ import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.ArrayUtil;
 import org.hsqldb.lib.HsqlArrayList;
-import org.hsqldb.lib.HsqlList;
+import org.hsqldb.lib.List;
 import org.hsqldb.lib.LongDeque;
 import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.map.ValuePool;
@@ -48,7 +48,7 @@ import org.hsqldb.types.Type;
  * Parser for DML statements
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.1
+ * @version 2.6.0
  * @since 1.9.0
  */
 public class ParserDML extends ParserDQL {
@@ -120,11 +120,11 @@ public class ParserDML extends ParserDQL {
                 read();
                 readThis(Tokens.VALUES);
 
-                insertExpressions = new Expression(OpTypes.ROW,
-                                                   new Expression[]{});
+                Expression row = new Expression(OpTypes.ROW,
+                                                Expression.emptyArray);
+
                 insertExpressions = new Expression(OpTypes.VALUELIST,
-                                                   new Expression[]{
-                                                       insertExpressions });
+                                                   new Expression[]{ row });
                 insertColumnCheckList = table.getNewColumnCheckList();
 
                 StatementDMQL cs = new StatementInsert(session, table,
@@ -217,7 +217,7 @@ public class ParserDML extends ParserDQL {
                     table.getColumnCheckList(insertColumnMap);
                 insertExpressions = XreadContextuallyTypedTable(colCount);
 
-                HsqlList unresolved =
+                List unresolved =
                     insertExpressions.resolveColumnReferences(session,
                         RangeGroup.emptyGroup, rangeGroups, null);
 
@@ -244,6 +244,7 @@ public class ParserDML extends ParserDQL {
                         Expression e = rowArgs[i];
                         ColumnSchema column =
                             baseTable.getColumn(insertColumnMap[i]);
+                        Type colType = column.getDataType();
 
                         if (column.isIdentity()) {
                             assignsToIdentityOrGenerated = true;
@@ -285,6 +286,17 @@ public class ParserDML extends ParserDQL {
                         if (e.isUnresolvedParam()) {
                             e.setAttributesAsColumn(column);
                         }
+
+                        // DYNAMIC_PARAM and PARAMETER expressions may have wider values
+                        if (e.opType != OpTypes.DEFAULT) {
+                            if (e.dataType == null || colType
+                                    .typeDataGroup != e.dataType
+                                    .typeDataGroup || colType.isArrayType()) {
+                                rowArgs[i] =
+                                    ExpressionOp.getCastExpression(session, e,
+                                                                   colType);
+                            }
+                        }
                     }
                 }
 
@@ -336,7 +348,7 @@ public class ParserDML extends ParserDQL {
 
                     exprList.toArray(updateExpressions);
                     resolveUpdateExpressions(table, rangeGroup,
-                                             updateColumnMap,
+                                             updateColumnMap, targets,
                                              updateExpressions, rangeGroups,
                                              valueRange);
                 }
@@ -450,7 +462,7 @@ public class ParserDML extends ParserDQL {
 
             exprList.toArray(updateExpressions);
             resolveUpdateExpressions(table, rangeGroup, updateColumnMap,
-                                     updateExpressions, rangeGroups,
+                                     targets, updateExpressions, rangeGroups,
                                      valueRange);
         }
 
@@ -748,7 +760,7 @@ public class ParserDML extends ParserDQL {
             sortAndSlice = XreadOrderByExpression();
         }
 
-        resolveUpdateExpressions(table, rangeGroup, columnMap,
+        resolveUpdateExpressions(table, rangeGroup, columnMap, targets,
                                  updateExpressions, rangeGroups, null);
 
         if (table != baseTable) {
@@ -818,7 +830,7 @@ public class ParserDML extends ParserDQL {
             RangeGroup rangeGroup) {
 
         Expression condition = XreadBooleanValueExpression();
-        HsqlList unresolved = condition.resolveColumnReferences(session,
+        List unresolved = condition.resolveColumnReferences(session,
             rangeGroup, rangeGroups, null);
 
         ExpressionColumn.checkColumnsResolved(unresolved);
@@ -836,13 +848,13 @@ public class ParserDML extends ParserDQL {
     }
 
     void resolveUpdateExpressions(Table targetTable, RangeGroup rangeGroup,
-                                  int[] columnMap,
+                                  int[] columnMap, Expression[] targets,
                                   Expression[] colExpressions,
                                   RangeGroup[] rangeGroups,
                                   RangeVariable valuesRange) {
 
-        HsqlList unresolved           = null;
-        int      enforcedDefaultIndex = -1;
+        List unresolved           = null;
+        int  enforcedDefaultIndex = -1;
 
         if (targetTable.hasIdentityColumn()
                 && targetTable.identitySequence.isAlways()) {
@@ -911,8 +923,16 @@ public class ParserDML extends ParserDQL {
                 }
 
                 if (e.isUnresolvedParam()) {
-                    e.setAttributesAsColumn(
-                        targetTable.getColumn(columnMap[i]));
+                    if (targets.length > i
+                            && targets[i].opType == OpTypes.ARRAY_ACCESS) {
+                        Type type = targetTable.getColumn(
+                            columnMap[i]).getDataType().collectionBaseType();
+
+                        e.setDataType(session, type);
+                    } else {
+                        e.setAttributesAsColumn(
+                            targetTable.getColumn(columnMap[i]));
+                    }
                 } else if (e.getType() == OpTypes.DEFAULT) {
 
                     //
@@ -1173,16 +1193,28 @@ public class ParserDML extends ParserDQL {
         LongDeque      updateColIndexList = new LongDeque();
         Expression[]   conditions         = new Expression[3];
         boolean        deleteFirst        = false;
+        int opOne = readMergeWhen(rangeGroups, fullRangeGroup,
+                                  updateColIndexList, insertColNames,
+                                  updateTargetSet, insertList, updateList,
+                                  targetRangeVars, sourceRange, conditions);
 
-        readMergeWhen(rangeGroups, fullRangeGroup, updateColIndexList,
-                      insertColNames, updateTargetSet, insertList, updateList,
-                      targetRangeVars, sourceRange, conditions);
-
-        // conditions[0], [1] and [2] are null (no action) or TRUE if there is no merge condition for the action
-        if (conditions[2] != null) {
+        if (opOne == StatementTypes.DELETE_WHERE) {
             deleteFirst = true;
         }
 
+        if (token.tokenType == Tokens.WHEN) {
+            int opTwo = readMergeWhen(rangeGroups, fullRangeGroup,
+                                      updateColIndexList, insertColNames,
+                                      updateTargetSet, insertList, updateList,
+                                      targetRangeVars, sourceRange,
+                                      conditions);
+
+            if (opTwo == StatementTypes.DELETE_WHERE
+                    && opOne == StatementTypes.INSERT) {
+                deleteFirst = true;
+            }
+        }
+
         if (token.tokenType == Tokens.WHEN) {
             readMergeWhen(rangeGroups, fullRangeGroup, updateColIndexList,
                           insertColNames, updateTargetSet, insertList,
@@ -1190,21 +1222,6 @@ public class ParserDML extends ParserDQL {
                           conditions);
         }
 
-        if (conditions[1] == null && conditions[2] != null) {
-            deleteFirst = true;
-        }
-
-        if (token.tokenType == Tokens.WHEN) {
-            readMergeWhen(rangeGroups, fullRangeGroup, updateColIndexList,
-                          insertColNames, updateTargetSet, insertList,
-                          updateList, targetRangeVars, sourceRange,
-                          conditions);
-        }
-
-        if (conditions[1] == null && conditions[2] != null) {
-            deleteFirst = true;
-        }
-
         if (insertList.size() > 0) {
             int colCount = insertColNames.size();
 
@@ -1259,10 +1276,11 @@ public class ParserDML extends ParserDQL {
             }
 
             resolveUpdateExpressions(table, fullRangeGroup, updateColumnMap,
-                                     updateExpressions, rangeGroups, null);
+                                     targets, updateExpressions, rangeGroups,
+                                     null);
         }
 
-        HsqlList unresolved = null;
+        List unresolved = null;
 
         unresolved = mergeCondition.resolveColumnReferences(session,
                 fullRangeGroup, rangeGroups, null);
@@ -1300,6 +1318,33 @@ public class ParserDML extends ParserDQL {
 
             ExpressionColumn.checkColumnsResolved(unresolved);
             insertExpression.resolveTypes(session, null);
+
+            Expression[] rowList = insertExpression.nodes;
+
+            for (int j = 0; j < rowList.length; j++) {
+                Expression[] rowArgs = rowList[j].nodes;
+
+                for (int i = 0; i < rowArgs.length; i++) {
+                    Expression   e       = rowArgs[i];
+                    ColumnSchema column  = table.getColumn(insertColumnMap[i]);
+                    Type         colType = column.getDataType();
+
+                    if (e.isUnresolvedParam()) {
+                        e.setAttributesAsColumn(column);
+                    }
+
+                    // DYNAMIC_PARAM and PARAMETER expressions may have wider values
+                    if (e.opType != OpTypes.DEFAULT) {
+                        if (e.dataType == null || colType.typeDataGroup != e
+                                .dataType.typeDataGroup || colType
+                                .isArrayType()) {
+                            rowArgs[i] =
+                                ExpressionOp.getCastExpression(session, e,
+                                                               colType);
+                        }
+                    }
+                }
+            }
         }
 
         StatementDMQL cs = new StatementDML(session, targets, sourceRange,
@@ -1316,23 +1361,18 @@ public class ParserDML extends ParserDQL {
 
     /**
      * Parses a WHEN clause from a MERGE statement. This can be either a
-     * WHEN MATCHED or WHEN NOT MATCHED clause, or both, and the appropriate
+     * WHEN MATCHED or WHEN NOT MATCHED clause, and the appropriate
      * values will be updated.
-     *
-     * If the var that is to hold the data is not null, then we already
-     * encountered this type of clause, which is only allowed once, and at least
-     * one is required.
      */
-    private void readMergeWhen(RangeGroup[] rangeGroups,
-                               RangeGroup rangeGroup,
-                               LongDeque updateColIndexList,
-                               OrderedHashSet insertColumnNames,
-                               OrderedHashSet updateTargetSet,
-                               HsqlArrayList insertExpressions,
-                               HsqlArrayList updateExpressions,
-                               RangeVariable[] targetRangeVars,
-                               RangeVariable sourceRangeVar,
-                               Expression[] conditions) {
+    private int readMergeWhen(RangeGroup[] rangeGroups, RangeGroup rangeGroup,
+                              LongDeque updateColIndexList,
+                              OrderedHashSet insertColumnNames,
+                              OrderedHashSet updateTargetSet,
+                              HsqlArrayList insertExpressions,
+                              HsqlArrayList updateExpressions,
+                              RangeVariable[] targetRangeVars,
+                              RangeVariable sourceRangeVar,
+                              Expression[] conditions) {
 
         Table      table       = targetRangeVars[0].rangeTable;
         int        columnCount = table.getColumnCount();
@@ -1361,6 +1401,8 @@ public class ParserDML extends ParserDQL {
                 readSetClauseList(rangeGroups, targetRangeVars,
                                   updateTargetSet, updateColIndexList,
                                   updateExpressions);
+
+                return StatementTypes.UPDATE_WHERE;
             } else {
                 if (conditions[2] != null) {
                     throw Error.error(ErrorCode.X_42547);
@@ -1373,6 +1415,8 @@ public class ParserDML extends ParserDQL {
                 conditions[2] = condition;
 
                 readThis(Tokens.DELETE);
+
+                return StatementTypes.DELETE_WHERE;
             }
         } else if (token.tokenType == Tokens.NOT) {
             if (insertExpressions.size() != 0) {
@@ -1418,6 +1462,8 @@ public class ParserDML extends ParserDQL {
             }
 
             insertExpressions.add(e);
+
+            return StatementTypes.INSERT;
         } else {
             throw unexpectedToken();
         }
@@ -1468,7 +1514,7 @@ public class ParserDML extends ParserDQL {
         }
 
         Expression expression = XreadValueExpression();
-        HsqlList unresolved = expression.resolveColumnReferences(session,
+        List unresolved = expression.resolveColumnReferences(session,
             RangeGroup.emptyGroup, rangeGroups, null);
 
         ExpressionColumn.checkColumnsResolved(unresolved);
@@ -1520,7 +1566,7 @@ public class ParserDML extends ParserDQL {
 
         compileContext.addProcedureCall(routine);
 
-        HsqlList unresolved = null;
+        List unresolved = null;
 
         for (int i = 0; i < arguments.length; i++) {
             Expression e = arguments[i];
@@ -1561,7 +1607,7 @@ public class ParserDML extends ParserDQL {
     void resolveReferencesAndTypes(RangeGroup rangeGroup,
                                    RangeGroup[] rangeGroups, Expression e) {
 
-        HsqlList unresolved = e.resolveColumnReferences(session, rangeGroup,
+        List unresolved = e.resolveColumnReferences(session, rangeGroup,
             rangeGroup.getRangeVariables().length, rangeGroups, null, false);
 
         ExpressionColumn.checkColumnsResolved(unresolved);
@@ -1574,7 +1620,7 @@ public class ParserDML extends ParserDQL {
     void resolveOuterReferencesAndTypes(RangeGroup[] rangeGroups,
                                         Expression e) {
 
-        HsqlList unresolved = e.resolveColumnReferences(session,
+        List unresolved = e.resolveColumnReferences(session,
             RangeGroup.emptyGroup, rangeGroups, null);
 
         ExpressionColumn.checkColumnsResolved(unresolved);
