@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2020, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,7 +62,7 @@ import org.hsqldb.types.Types;
  * Scans for SQL tokens.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.6.0
  * @since 1.9.0
  */
 public class Scanner {
@@ -187,9 +187,6 @@ public class Scanner {
 
     //
     private boolean hasNonSpaceSeparator;
-    private int     eolPosition;
-    private int     lineNumber;
-    private int     eolCode;
 
     //
     private static final int maxPooledStringLength =
@@ -227,8 +224,6 @@ public class Scanner {
         tokenPosition        = 0;
         limit                = sqlString.length();
         hasNonSpaceSeparator = false;
-        eolPosition          = -1;
-        lineNumber           = 1;
 
         token.reset();
 
@@ -321,7 +316,7 @@ public class Scanner {
     }
 
     public int getLineNumber() {
-        return lineNumber;
+        return Scanner.countEndOfLines(sqlString, tokenPosition) + 1;
     }
 
     int getTokenPosition() {
@@ -370,7 +365,7 @@ public class Scanner {
     }
 
     /**
-     * returns hex value of a hex character, or 16 if not a hex character
+     * returns hex value of a hex character, or -1 if not a hex character
      */
     static int getHexValue(int c) {
 
@@ -485,8 +480,6 @@ public class Scanner {
             // no end quote
             token.tokenType   = Tokens.X_MALFORMED_BINARY_STRING;
             token.isMalformed = true;
-
-            return;
         }
     }
 
@@ -1255,8 +1248,6 @@ public class Scanner {
                 hasNonSpaceSeparator = true;
                 result               = true;
 
-                setLineNumber(c);
-
                 continue;
             }
 
@@ -1266,32 +1257,13 @@ public class Scanner {
         return result;
     }
 
-    private void setLineNumber(int c) {
-
-        if (c == '\r' || c == '\n') {
-            if (currentPosition == eolPosition + 1) {
-                if (c == '\n' && eolCode != c) {
-
-                    //
-                } else {
-                    lineNumber++;
-                }
-            } else {
-                lineNumber++;
-            }
-
-            eolPosition = currentPosition;
-            eolCode     = c;
-        }
-    }
-
-    private static int countEndOfLines(String s) {
+    private static int countEndOfLines(String s, int toPosition) {
 
         int eolPos    = -2;
         int eolCode   = 0;
         int lineCount = 0;
 
-        for (int i = 0; i < s.length(); i++) {
+        for (int i = 0; i < toPosition; i++) {
             int c = s.charAt(i);
 
             if (c == '\r' || c == '\n') {
@@ -1491,7 +1463,7 @@ public class Scanner {
                 if (charAt(currentPosition + 1) == ':') {
                     currentPosition   += 2;
                     token.tokenString = Tokens.T_DOUBLE_COLON;
-                    token.tokenType   = Tokens.COLON;
+                    token.tokenType   = Tokens.DOUBLE_COLON_OP;
                     token.isDelimiter = true;
 
                     return;
@@ -1789,7 +1761,7 @@ public class Scanner {
 
             case '_' :
 
-                /**
+                /*
                  * @todo 1.9.0 - review following
                  * identifier chain must not have catalog identifier
                  * character set specification to be included in the token.dataType
@@ -1806,7 +1778,7 @@ public class Scanner {
 
                 if (token.tokenType != Tokens.X_IDENTIFIER) {
 
-                    /** @todo 1.9.0 - review message malformed character set identifier */
+                    /* @todo 1.9.0 - review message malformed character set identifier */
                     token.tokenType   = Tokens.X_MALFORMED_STRING;
                     token.isMalformed = true;
 
@@ -1818,7 +1790,7 @@ public class Scanner {
                 if (charAt(currentPosition) == '\'') {
                     if (token.namePrePrefix != null) {
 
-                        /** @todo 1.9.0 - review message malformed character set identifier */
+                        /* @todo 1.9.0 - review message malformed character set identifier */
                         token.tokenType   = Tokens.X_MALFORMED_STRING;
                         token.isMalformed = true;
 
@@ -1880,7 +1852,6 @@ public class Scanner {
         String comment = sqlString.substring(currentPosition + 2, pos);
 
         currentPosition = pos + 2;
-        lineNumber      += countEndOfLines(comment);
 
         return true;
     }
@@ -1901,7 +1872,6 @@ public class Scanner {
         token.tokenString = sqlString.substring(currentPosition + 2, pos);
         token.tokenType   = Tokens.X_REMARK;
         token.isDelimiter = true;
-        lineNumber        += countEndOfLines(token.tokenString);
 
         return true;
     }
@@ -2069,7 +2039,7 @@ public class Scanner {
         return new TimestampData(seconds);
     }
 
-    /**
+    /*
      * @todo 1.9.0 - review the following
      *      - misses nano fractions
      *      - misses displacement
@@ -2469,14 +2439,39 @@ public class Scanner {
         scanToken();
         scanWhitespace();
 
-        if (token.tokenType == Tokens.PLUS_OP) {
-            scanToken();
-            scanWhitespace();
-        } else if (token.tokenType == Tokens.MINUS_OP) {
-            minus = true;
+        switch (token.tokenType) {
 
-            scanToken();
-            scanWhitespace();
+            case Tokens.MINUS_OP :
+                minus = true;
+
+                scanToken();
+                scanWhitespace();
+
+                if (token.tokenType == Tokens.INFINITY) {
+                    minus           = false;
+                    token.tokenType = Tokens.X_VALUE;
+                    token.dataType  = Type.SQL_DOUBLE;
+                    token.tokenValue =
+                        Double.valueOf(Double.NEGATIVE_INFINITY);
+                }
+                break;
+
+            case Tokens.PLUS_OP :
+                scanToken();
+                scanWhitespace();
+                break;
+
+            case Tokens.NAN :
+                token.tokenType  = Tokens.X_VALUE;
+                token.dataType   = Type.SQL_DOUBLE;
+                token.tokenValue = Double.valueOf(Double.NaN);
+                break;
+
+            case Tokens.INFINITY :
+                token.tokenType  = Tokens.X_VALUE;
+                token.dataType   = Type.SQL_DOUBLE;
+                token.tokenValue = Double.valueOf(Double.POSITIVE_INFINITY);
+                break;
         }
 
         if (!hasNonSpaceSeparator && token.tokenType == Tokens.X_VALUE
