@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2020, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -75,7 +75,7 @@ import org.hsqldb.types.TypedComparator;
  * Implementation of SQL sessions.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.1
+ * @version 2.6.0
  * @since 1.7.0
  */
 public class Session implements SessionInterface {
@@ -125,9 +125,8 @@ public class Session implements SessionInterface {
     private int        sessionMaxRows;
     int                sessionOptimization = 8;
     private final long sessionId;
-    int                sessionTxId = -1;
     private boolean    ignoreCase;
-    private long       sessionStartTimestamp;
+    private final long connectTime = System.currentTimeMillis();
 
     // internal connection
     private JDBCConnection intConnection;
@@ -174,7 +173,7 @@ public class Session implements SessionInterface {
         this.zoneString             = zoneString;
         this.sessionTimeZoneSeconds = timeZoneSeconds;
         this.timeZoneSeconds        = timeZoneSeconds;
-        rowActionList               = new HsqlArrayList(32, true);
+        rowActionList               = new HsqlArrayList(128, true);
         waitedSessions              = new OrderedHashSet();
         waitingSessions             = new OrderedHashSet();
         tempSet                     = new OrderedHashSet();
@@ -197,10 +196,9 @@ public class Session implements SessionInterface {
         setResultMemoryRowCount(database.getResultMaxMemoryRows());
         resetSchema();
 
-        sessionData           = new SessionData(database, this);
-        statementManager      = new StatementManager(database);
-        timeoutManager        = new TimeoutManager();
-        sessionStartTimestamp = System.currentTimeMillis();
+        sessionData      = new SessionData(database, this);
+        statementManager = new StatementManager(database);
+        timeoutManager   = new TimeoutManager();
     }
 
     void resetSchema() {
@@ -413,7 +411,6 @@ public class Session implements SessionInterface {
 
     /**
      * This is used for reading - writing to existing tables.
-     * @throws  HsqlException
      */
     void checkReadWrite() {
 
@@ -424,7 +421,6 @@ public class Session implements SessionInterface {
 
     /**
      * This is used for creating new database objects such as tables.
-     * @throws  HsqlException
      */
     void checkDDLWrite() {
 
@@ -444,7 +440,6 @@ public class Session implements SessionInterface {
      *
      * @param  table the table of the row
      * @param  row the deleted row
-     * @throws  HsqlException
      */
     public void addDeleteAction(Table table, PersistentStore store, Row row,
                                 int[] changedColumns) {
@@ -490,7 +485,6 @@ public class Session implements SessionInterface {
      *  Setter for the autocommit attribute.
      *
      * @param  autocommit the new value
-     * @throws  HsqlException
      */
     public synchronized void setAutoCommit(boolean autocommit) {
 
@@ -552,7 +546,7 @@ public class Session implements SessionInterface {
 
     public synchronized void startPhasedTransaction() {}
 
-    /**
+    /*
      * @todo - fredt - for two phased pre-commit - after this call, further
      * state changing calls should fail
      */
@@ -573,8 +567,6 @@ public class Session implements SessionInterface {
 
     /**
      * Commits any uncommitted transaction this Session may have open
-     *
-     * @throws  HsqlException
      */
     public synchronized void commit(boolean chain) {
 
@@ -609,8 +601,6 @@ public class Session implements SessionInterface {
 
     /**
      * Rolls back any uncommitted transaction this Session may have open.
-     *
-     * @throws  HsqlException
      */
     public synchronized void rollback(boolean chain) {
 
@@ -706,7 +696,6 @@ public class Session implements SessionInterface {
      *  name of an existing one replaces the old SAVEPOINT.
      *
      * @param  name name of the savepoint
-     * @throws  HsqlException if there is no current transaction
      */
     public synchronized void savepoint(String name) {
 
@@ -728,7 +717,6 @@ public class Session implements SessionInterface {
      *  Performs a partial transaction ROLLBACK to savepoint.
      *
      * @param  name name of savepoint
-     * @throws  HsqlException
      */
     public synchronized void rollbackToSavepoint(String name) {
 
@@ -747,8 +735,6 @@ public class Session implements SessionInterface {
 
     /**
      * Performs a partial transaction ROLLBACK of current savepoint level.
-     *
-     * @throws  HsqlException
      */
     public synchronized void rollbackToSavepoint() {
 
@@ -772,7 +758,6 @@ public class Session implements SessionInterface {
      * Releases a savepoint
      *
      * @param  name name of savepoint
-     * @throws  HsqlException if name does not correspond to a savepoint
      */
     public synchronized void releaseSavepoint(String name) {
 
@@ -784,8 +769,8 @@ public class Session implements SessionInterface {
         }
 
         while (sessionContext.savepoints.size() > index) {
-            sessionContext.savepoints.remove(sessionContext.savepoints.size()
-                                             - 1);
+            sessionContext.savepoints.removeEntry(
+                sessionContext.savepoints.size() - 1);
             sessionContext.savepointTimestamps.removeLast();
         }
     }
@@ -906,8 +891,6 @@ public class Session implements SessionInterface {
 
 // campbell-burnet@users 20020810 metadata 1.7.2
 //----------------------------------------------------------------
-    private final long connectTime = System.currentTimeMillis();
-
 // more efficient for MetaData concerns than checkAdmin
 
     /**
@@ -1310,7 +1293,7 @@ public class Session implements SessionInterface {
 
             try {
 
-                /** special autocommit for backward compatibility */
+                /* special autocommit for backward compatibility */
                 commit(false);
             } catch (HsqlException e) {
                 database.logger.logInfoEvent("Exception at commit");
@@ -2022,7 +2005,7 @@ public class Session implements SessionInterface {
         if (result.getType() == ResultConstants.SQLCANCEL) {
             if (result.getSessionRandomID() == randomId) {
                 database.txManager.resetSession(
-                    null, this, Long.MAX_VALUE,
+                    this, this, statementStartTimestamp,
                     TransactionManager.resetSessionStatement);
             }
         }
@@ -2344,41 +2327,46 @@ public class Session implements SessionInterface {
     // timeouts
     class TimeoutManager {
 
-        AtomicInteger currentTimeout = new AtomicInteger();
-        volatile long checkTimestamp;
+        AtomicInteger    currentTimeout = new AtomicInteger();
+        volatile long    checkTimestampSCN;
+        volatile boolean hasTimeout = false;
 
         void startTimeout(int timeout) {
 
-            currentTimeout.set(timeout);
-
             if (timeout == 0) {
                 return;
             }
 
-            checkTimestamp = Session.this.statementStartTimestamp;
+            hasTimeout = true;
+
+            currentTimeout.set(timeout);
+            checkTimestampSCN = statementStartTimestamp;
 
             database.timeoutRunner.addSession(Session.this);
         }
 
-        boolean endTimeout() {
+        void endTimeout() {
 
-            currentTimeout.set(0);
+            if (hasTimeout) {
+                currentTimeout.set(0);
 
-            return true;
+                checkTimestampSCN = 0;
+                hasTimeout        = false;
+            }
         }
 
         public boolean checkTimeout() {
 
-            if (currentTimeout.get() == 0) {
-                return true;
+            if (!hasTimeout || checkTimestampSCN != statementStartTimestamp) {
+                return false;
             }
 
             int result = currentTimeout.decrementAndGet();
 
-            if (result <= 0) {
+            if (result < 0) {
                 currentTimeout.set(0);
                 database.txManager.resetSession(
-                    null, Session.this, checkTimestamp,
+                    Session.this, Session.this, checkTimestampSCN,
                     TransactionManager.resetSessionStatement);
 
                 return true;
