@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2020, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,10 +45,10 @@ import java.util.HashMap;
 import java.util.Map;
 
 import org.hsqldb.cmdline.sqltool.Token;
-import org.hsqldb.lib.FrameworkLogger;
-import org.hsqldb.lib.RCData;
+import org.hsqldb.cmdline.libclone.FrameworkLogger;
+import org.hsqldb.cmdline.utilclone.RCData;
 
-/* $Id: SqlTool.java 6140 2020-06-24 22:00:54Z fredt $ */
+/* $Id: SqlTool.java 6266 2021-01-25 16:08:06Z fredt $ */
 
 /**
  * A command-line JDBC SQL tool supporting both interactive and
@@ -71,7 +71,7 @@ import org.hsqldb.lib.RCData;
  * @see #objectMain(String[])
  * @see SqlFile
  * @see org.hsqldb.sample.SqlFileEmbedder
- * @version $Revision: 6140 $, $Date: 2020-06-24 23:00:54 +0100 (Wed, 24 Jun 2020) $
+ * @version $Revision: 6266 $, $Date: 2021-01-25 16:08:06 +0000 (Mon, 25 Jan 2021) $
  * @author Blaine Simpson (blaine dot simpson at admc dot com)
  */
 public class SqlTool {
@@ -80,7 +80,7 @@ public class SqlTool {
     public static final String DEFAULT_RCFILE =
         System.getProperty("user.home") + "/sqltool.rc";
     // N.b. the following are static!
-    private static final String revString = "$Revision: 6140 $";
+    private static final String revString = "$Revision: 6266 $";
     private static final int revStringLength = revString.length();
     private static final String  revnum =
             (revStringLength - " $".length() > "$Revision: ".length())
