@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2020, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,26 +32,27 @@
 package org.hsqldb.lib;
 
 import java.lang.reflect.Array;
+import java.util.Arrays;
 
 /**
  * Collection of static methods for operations on arrays
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.6.0
  * @since 1.7.2
  */
-public class ArrayUtil {
+public final class ArrayUtil {
 
-    public static final int        CLASS_CODE_BYTE    = 'B';
-    public static final int        CLASS_CODE_CHAR    = 'C';
-    public static final int        CLASS_CODE_DOUBLE  = 'D';
-    public static final int        CLASS_CODE_FLOAT   = 'F';
-    public static final int        CLASS_CODE_INT     = 'I';
-    public static final int        CLASS_CODE_LONG    = 'J';
-    public static final int        CLASS_CODE_OBJECT  = 'L';
-    public static final int        CLASS_CODE_SHORT   = 'S';
-    public static final int        CLASS_CODE_BOOLEAN = 'Z';
-    private static IntValueHashMap classCodeMap       = new IntValueHashMap();
+    public static final int              CLASS_CODE_BYTE    = 'B';
+    public static final int              CLASS_CODE_CHAR    = 'C';
+    public static final int              CLASS_CODE_DOUBLE  = 'D';
+    public static final int              CLASS_CODE_FLOAT   = 'F';
+    public static final int              CLASS_CODE_INT     = 'I';
+    public static final int              CLASS_CODE_LONG    = 'J';
+    public static final int              CLASS_CODE_OBJECT  = 'L';
+    public static final int              CLASS_CODE_SHORT   = 'S';
+    public static final int              CLASS_CODE_BOOLEAN = 'Z';
+    private static final IntValueHashMap classCodeMap = new IntValueHashMap(16);
 
     static {
         classCodeMap.put(byte.class, ArrayUtil.CLASS_CODE_BYTE);
@@ -66,7 +67,11 @@ public class ArrayUtil {
     }
 
     /**
-     * Returns a distinct int code for each primitive type and for all Object types.
+     * Returns a distinct int code for each primitive type and for all Object
+     * types.
+     *
+     * @param cla Class
+     * @return int
      */
     static int getClassCode(Class cla) {
 
@@ -79,6 +84,11 @@ public class ArrayUtil {
 
     /**
      * Clears an area of the given array of the given type.
+     *
+     * @param type int
+     * @param data Object
+     * @param from int
+     * @param to int
      */
     public static void clearArray(int type, Object data, int from, int to) {
 
@@ -87,81 +97,63 @@ public class ArrayUtil {
             case ArrayUtil.CLASS_CODE_BYTE : {
                 byte[] array = (byte[]) data;
 
-                while (--to >= from) {
-                    array[to] = 0;
-                }
+                Arrays.fill(array, from, to, (byte) 0);
 
                 return;
             }
             case ArrayUtil.CLASS_CODE_CHAR : {
                 char[] array = (char[]) data;
 
-                while (--to >= from) {
-                    array[to] = 0;
-                }
+                Arrays.fill(array, from, to, (char) 0);
 
                 return;
             }
             case ArrayUtil.CLASS_CODE_SHORT : {
                 short[] array = (short[]) data;
 
-                while (--to >= from) {
-                    array[to] = 0;
-                }
+                Arrays.fill(array, from, to, (short) 0);
 
                 return;
             }
             case ArrayUtil.CLASS_CODE_INT : {
                 int[] array = (int[]) data;
 
-                while (--to >= from) {
-                    array[to] = 0;
-                }
+                Arrays.fill(array, from, to, 0);
 
                 return;
             }
             case ArrayUtil.CLASS_CODE_LONG : {
                 long[] array = (long[]) data;
 
-                while (--to >= from) {
-                    array[to] = 0;
-                }
+                Arrays.fill(array, from, to, 0L);
 
                 return;
             }
             case ArrayUtil.CLASS_CODE_FLOAT : {
                 float[] array = (float[]) data;
 
-                while (--to >= from) {
-                    array[to] = 0;
-                }
+                Arrays.fill(array, from, to, 0);
 
                 return;
             }
             case ArrayUtil.CLASS_CODE_DOUBLE : {
                 double[] array = (double[]) data;
 
-                while (--to >= from) {
-                    array[to] = 0;
-                }
+                Arrays.fill(array, from, to, 0);
 
                 return;
             }
             case ArrayUtil.CLASS_CODE_BOOLEAN : {
                 boolean[] array = (boolean[]) data;
 
-                while (--to >= from) {
-                    array[to] = false;
-                }
+                Arrays.fill(array, from, to, false);
 
                 return;
             }
             default : {
                 Object[] array = (Object[]) data;
 
-                while (--to >= from) {
-                    array[to] = null;
-                }
+                Arrays.fill(array, from, to, null);
 
                 return;
             }
@@ -211,6 +203,8 @@ public class ArrayUtil {
 
     /**
      * Basic sort for small arrays of int.
+     *
+     * @param array int[]
      */
     public static void sortArray(int[] array) {
 
@@ -232,7 +226,11 @@ public class ArrayUtil {
     }
 
     /**
-     *  Basic find for small arrays of Object.
+     * Basic find for small arrays of Object.
+     *
+     * @param array Object[]
+     * @param object Object
+     * @return int
      */
     public static int find(Object[] array, Object object) {
 
@@ -252,7 +250,11 @@ public class ArrayUtil {
     }
 
     /**
-     *  Find index of value in small array.
+     * Finds index of value in small array.
+     *
+     * @param array int[]
+     * @param value int
+     * @return int
      */
     public static int find(int[] array, int value) {
 
@@ -265,6 +267,25 @@ public class ArrayUtil {
         return -1;
     }
 
+    /**
+     * Finds index of value in the first count elements of small array.
+     *
+     * @param array int[]
+     * @param count int
+     * @param value int
+     * @return int
+     */
+    public static int find(int[] array, int count, int value) {
+
+        for (int i = 0; i < count; i++) {
+            if (array[i] == value) {
+                return i;
+            }
+        }
+
+        return -1;
+    }
+
     public static int find(short[] array, int value) {
 
         for (int i = 0; i < array.length; i++) {
@@ -288,7 +309,11 @@ public class ArrayUtil {
     }
 
     /**
-     *  Finds the first element of the array that is not equal to the given value.
+     * Finds the first element of the array that is not equal to the given value.
+     *
+     * @param array int[]
+     * @param value int
+     * @return int
      */
     public static int findNot(int[] array, int value) {
 
@@ -305,40 +330,24 @@ public class ArrayUtil {
      * Returns true if arra and arrb contain the same set of integers, not
      * necessarily in the same order. This implies the arrays are of the same
      * length.
+     *
+     * @param arra int[]
+     * @param arrb int[]
+     * @return boolean
      */
     public static boolean areEqualSets(int[] arra, int[] arrb) {
         return arra.length == arrb.length
                && ArrayUtil.haveEqualSets(arra, arrb, arra.length);
     }
 
-    /**
-     * For full == true returns true if arra and arrb are identical (have the
-     * same length and contain the same integers in the same sequence).
-     *
-     * For full == false returns the result
-     * of haveEqualArrays(arra,arrb,count)
-     *
-     * For full == true, the array lengths must be the same as count
-     *
-     */
-    public static boolean areEqual(int[] arra, int[] arrb, int count,
-                                   boolean full) {
-
-        if (ArrayUtil.haveEqualArrays(arra, arrb, count)) {
-            if (full) {
-                return arra.length == arrb.length && count == arra.length;
-            }
-
-            return true;
-        }
-
-        return false;
-    }
-
     /**
      * Returns true if the first count elements of arra and arrb are identical
      * sets of integers (not necessarily in the same order).
      *
+     * @param arra int[]
+     * @param arrb int[]
+     * @param count int
+     * @return boolean
      */
     public static boolean haveEqualSets(int[] arra, int[] arrb, int count) {
 
@@ -366,6 +375,10 @@ public class ArrayUtil {
      * Returns true if the first count elements of arra and arrb are identical
      * subarrays of integers
      *
+     * @param arra int[]
+     * @param arrb int[]
+     * @param count int
+     * @return boolean
      */
     public static boolean haveEqualArrays(int[] arra, int[] arrb, int count) {
 
@@ -386,6 +399,10 @@ public class ArrayUtil {
      * Returns true if the first count elements of arra and arrb are identical
      * subarrays of Objects
      *
+     * @param arra Object[]
+     * @param arrb Object[]
+     * @param count int
+     * @return boolean
      */
     public static boolean haveEqualArrays(Object[] arra, Object[] arrb,
                                           int count) {
@@ -406,9 +423,13 @@ public class ArrayUtil {
     }
 
     /**
-     * Returns true if arra and arrb share any element. <p>
+     * Returns true if arra and arrb share any element.
      *
-     * Used for checks for any overlap between two arrays of column indexes.
+     * <p> Used for checks for any overlap between two arrays of column indexes.
+     *
+     * @param arra int[]
+     * @param arrb int[]
+     * @return boolean
      */
     public static boolean haveCommonElement(int[] arra, int[] arrb) {
 
@@ -435,22 +456,21 @@ public class ArrayUtil {
      *
      * Used to find the overlap between two arrays of column indexes.
      * Ordering of the result arrays will be the same as in array
-     * a. The method assumes that each index is only listed
+     * arra. The method assumes that each index is only listed
      * once in the two input arrays.
      * <p>
      * e.g.
-     * </p>
-     * <code>
-     * <table width="90%" bgcolor="lightblue">
-     * <tr><td colspane="3">The arrays</td></tr>
+     * <table>
+     * <caption>Example</caption>
+     * <tr><td colspan="3">The arrays</td></tr>
      * <tr><td>int []arra</td><td>=</td><td>{2,11,5,8}</td></tr>
      * <tr><td>int []arrb</td><td>=</td><td>{20,8,10,11,28,12}</td></tr>
-     * <tr><td colspane="3">will result in:</td></tr>
+     * <tr><td colspan="3">will result in:</td></tr>
      * <tr><td>int []arrc</td><td>=</td><td>{11,8}</td></tr>
      * </table>
      *
-     * @param arra int[]; first column indexes
-     * @param arrb int[]; second column indexes
+     * @param arra int[] first column indexes
+     * @param arrb int[] second column indexes
      * @return int[] common indexes or <code>null</code> if there is no overlap.
      */
     public static int[] commonElements(int[] arra, int[] arrb) {
@@ -479,17 +499,17 @@ public class ArrayUtil {
      * Returns the number of elements shared between the two arrays containing
      * sets.<p>
      *
-     * Return the number of elements shared by two column index arrays.
+     * Returns the number of elements shared by two column index arrays.
      * This method assumes that each of these arrays contains a set (each
      * element index is listed only once in each index array). Otherwise the
      * returned number will NOT represent the number of unique column indexes
      * shared by both index array.
      *
-     * @param arra int[]; first array of column indexes.
+     * @param arra int[] first array of column indexes.
      *
-     * @param arrb int[]; second array of column indexes
+     * @param arrb int[] second array of column indexes
      *
-     * @return int; number of elements shared by <code>a</code> and <code>b</code>
+     * @return int number of elements shared by <code>arra</code> and <code>arrb</code>
      */
     public static int countCommonElements(int[] arra, int[] arrb) {
 
@@ -529,6 +549,11 @@ public class ArrayUtil {
     /**
      * Returns the count of elements in arra from position start that are
      * sequentially equal to the elements of arrb.
+     *
+     * @param arra byte[]
+     * @param start int
+     * @param arrb byte[]
+     * @return int
      */
     public static int countSameElements(byte[] arra, int start, byte[] arrb) {
 
@@ -553,6 +578,11 @@ public class ArrayUtil {
     /**
      * Returns the count of elements in arra from position start that are
      * sequentially equal to the elements of arrb.
+     *
+     * @param arra char[]
+     * @param start int
+     * @param arrb char[]
+     * @return int
      */
     public static int countSameElements(char[] arra, int start, char[] arrb) {
 
@@ -577,6 +607,11 @@ public class ArrayUtil {
     /**
      * Returns the count of elements in arra from position start that are
      * sequentially equal to the elements of arrb.
+     *
+     * @param arra int[]
+     * @param start int
+     * @param arrb int[]
+     * @return int
      */
     public static int countSameElements(int[] arra, int start, int[] arrb) {
 
@@ -600,6 +635,10 @@ public class ArrayUtil {
 
     /**
      * Returns the count of elements in arra that are smaller than the value.
+     *
+     * @param arra int[]
+     * @param value int
+     * @return int
      */
     public static int countSmallerElements(int[] arra, int value) {
 
@@ -617,6 +656,10 @@ public class ArrayUtil {
     /**
      * Returns the count of elements in arra that are smaller/equal than the
      * value.
+     *
+     * @param arra int[]
+     * @param value int
+     * @return int
      */
     public static int countSmallerEqualElements(int[] arra, int value) {
 
@@ -633,36 +676,72 @@ public class ArrayUtil {
 
     /**
      * Returns an array that contains all the elements of the two arrays.
+     * Each array contains a set.
+     *
+     * @param arra int[] containing unique vlaues
+     * @param arrb int[] containing unique values
+     * @return int[]
      */
     public static int[] union(int[] arra, int[] arrb) {
 
-        int newSize = arra.length + arrb.length
-                      - ArrayUtil.countCommonElements(arra, arrb);
+        int commonSize  = ArrayUtil.countCommonElements(arra, arrb);
 
-        if (newSize > arra.length && newSize > arrb.length) {
-            int[] arrn = (int[]) ArrayUtil.resizeArray(arrb, newSize);
-            int   pos  = arrb.length;
+        if (commonSize == arrb.length) {
+            return arra;
+        }
+
+        if (commonSize == arra.length) {
+            return arrb;
+        }
+
+        int   newSize = arra.length + arrb.length - commonSize;
+        int[] arrn    = Arrays.copyOf(arra, newSize);
+        int   pos     = arra.length;
 
             mainloop:
-            for (int i = 0; i < arra.length; i++) {
-                for (int j = 0; j < arrb.length; j++) {
-                    if (arra[i] == arrb[j]) {
+            for (int i = 0; i < arrb.length; i++) {
+                for (int j = 0; j < arra.length; j++) {
+                    if (arrb[i] == arra[j]) {
                         continue mainloop;
                     }
                 }
 
-                arrn[pos++] = arra[i];
+                arrn[pos++] = arrb[i];
             }
 
             return arrn;
-        }
-
-        return arra.length > arrb.length ? arra
-                                         : arrb;
     }
 
     /**
-     * Returns the index of the first occurrence of arrb in arra. Or -1 if not found.
+     * Returns an array that contains all the elements of the two arrays.
+     *
+     * @param arra int[]
+     * @param arrb int[]
+     * @return int[]
+     */
+    public static int[] concat(int[] arra, int[] arrb) {
+        int newSize = arra.length + arrb.length;
+
+        int[] arrn = Arrays.copyOf(arra, newSize);
+        int   pos  = arra.length;
+
+        for (int i = 0; i < arrb.length; i++) {
+            arrn[pos++] = arrb[i];
+        }
+
+        return arrn;
+
+    }
+
+    /**
+     * Returns the index of the first occurrence of arrb in arra. Or -1 if not
+     * found.
+     *
+     * @param arra byte[]
+     * @param start int
+     * @param limit int
+     * @param arrb byte[]
+     * @return int
      */
     public static int find(byte[] arra, int start, int limit, byte[] arrb) {
 
@@ -690,6 +769,12 @@ public class ArrayUtil {
     /**
      * Returns an index into arra (or -1) where the character is not in the
      * charset byte array.
+     *
+     * @param arra byte[]
+     * @param start int
+     * @param limit int
+     * @param byteSet byte[]
+     * @return int
      */
     public static int findNotIn(byte[] arra, int start, int limit,
                                 byte[] byteSet) {
@@ -709,8 +794,14 @@ public class ArrayUtil {
     }
 
     /**
-     * Returns an index into arra (or -1) where the character is in the
-     * byteSet byte array.
+     * Returns an index into arra (or -1) where the character is in the byteSet
+     * byte array.
+     *
+     * @param arra byte[]
+     * @param start int
+     * @param limit int
+     * @param byteSet byte[]
+     * @return int
      */
     public static int findIn(byte[] arra, int start, int limit,
                              byte[] byteSet) {
@@ -728,6 +819,13 @@ public class ArrayUtil {
 
     /**
      * Returns the index of b or c in arra. Or -1 if not found.
+     *
+     * @param arra byte[]
+     * @param start int
+     * @param limit int
+     * @param b int
+     * @param c int
+     * @return int
      */
     public static int find(byte[] arra, int start, int limit, int b, int c) {
 
@@ -744,6 +842,9 @@ public class ArrayUtil {
 
     /**
      * Set elements of arrb true if their indexes appear in arrb.
+     *
+     * @param arrb boolean[]
+     * @return int[]
      */
     public static int[] booleanArrayToIntIndexes(boolean[] arrb) {
 
@@ -770,6 +871,9 @@ public class ArrayUtil {
 
     /**
      * Set elements of arrb true if their indexes appear in arrb.
+     *
+     * @param arra int[]
+     * @param arrb boolean[]
      */
     public static void intIndexesToBooleanArray(int[] arra, boolean[] arrb) {
 
@@ -782,6 +886,10 @@ public class ArrayUtil {
 
     /**
      * Return array of indexes of boolean elements that are true.
+     *
+     * @param arra int[]
+     * @param arrb boolean[]
+     * @return int
      */
     public static int countStartIntIndexesInBooleanArray(int[] arra,
             boolean[] arrb) {
@@ -807,9 +915,14 @@ public class ArrayUtil {
     }
 
     /**
-     * Returns true if all indexes and no other positions are true in
-     * arrb.
+     * Returns true if all indexes and no other positions are true in arrb. arra
+     * must have no duplicates.
+     *
      * arra must have no duplicates.
+     *
+     * @param arra int[]
+     * @param arrb boolean[]
+     * @return boolean
      */
     public static boolean areAllIntIndexesAsBooleanArray(int[] arra,
             boolean[] arrb) {
@@ -852,8 +965,12 @@ public class ArrayUtil {
     }
 
     /**
-     * Return true if for each true element in arrb, the corresponding
-     * element in arra is true
+     * Return true if for each true element in arrb, the corresponding element
+     * in arra is true
+     *
+     * @param arra boolean[]
+     * @param arrb boolean[]
+     * @return boolean
      */
     public static boolean containsAllTrueElements(boolean[] arra,
             boolean[] arrb) {
@@ -869,6 +986,9 @@ public class ArrayUtil {
 
     /**
      * Return count of true elements in array
+     *
+     * @param arra boolean[]
+     * @return int
      */
     public static int countTrueElements(boolean[] arra) {
 
@@ -886,6 +1006,10 @@ public class ArrayUtil {
     /**
      * Determines if the array has a null column for any of the positions given
      * in the rowColMap array.
+     *
+     * @param array Object[]
+     * @param columnMap int[]
+     * @return boolean
      */
     public static boolean hasNull(Object[] array, int[] columnMap) {
 
@@ -914,16 +1038,26 @@ public class ArrayUtil {
     }
 
     /**
-     * Returns true if arra from position start contains all elements of arrb
-     * in sequential order.
+     * Returns true if arra from position start contains all elements of arrb in
+     * sequential order.
+     *
+     * @param arra byte[]
+     * @param start int
+     * @param arrb byte[]
+     * @return boolean
      */
     public static boolean containsAt(byte[] arra, int start, byte[] arrb) {
         return countSameElements(arra, start, arrb) == arrb.length;
     }
 
     /**
-     * Returns the count of elements in arra from position start that are
-     * among the elements of arrb. Stops at any element not in arrb.
+     * Returns the count of elements in arra from position start that are among
+     * the elements of arrb. Stops at any element not in arrb.
+     *
+     * @param arra byte[]
+     * @param start int
+     * @param arrb byte[]
+     * @return int
      */
     public static int countStartElementsAt(byte[] arra, int start,
                                            byte[] arrb) {
@@ -947,8 +1081,13 @@ public class ArrayUtil {
     }
 
     /**
-     * Returns true if arra from position start contains all elements of arrb
-     * in sequential order.
+     * Returns true if arra from position start contains all elements of arrb in
+     * sequential order.
+     *
+     * @param arra char[]
+     * @param start int
+     * @param arrb char[]
+     * @return boolean
      */
     public static boolean containsAt(char[] arra, int start, char[] arrb) {
         return countSameElements(arra, start, arrb) == arrb.length;
@@ -958,6 +1097,10 @@ public class ArrayUtil {
      * Returns the count of elements in arra from position start that are not
      * among the elements of arrb.
      *
+     * @param arra byte[]
+     * @param start int
+     * @param arrb byte[]
+     * @return int
      */
     public static int countNonStartElementsAt(byte[] arra, int start,
             byte[] arrb) {
@@ -980,9 +1123,18 @@ public class ArrayUtil {
 
     /**
      * Byte arrays source and dest each begin at an offset in the common space.
-     * If there is an overlap between dest and the first sourceLength elements of
-     * the source, the overlapping elements are copied to dest. Returns count
+     * If there is an overlap between dest and the first sourceLength elements
+     * of the source, the overlapping elements are copied to dest. Returns count
      * of copied bytes.
+     *
+     * @param sourceOffset long
+     * @param source byte[]
+     * @param sourceOff int
+     * @param sourceLength int
+     * @param destOffset long
+     * @param dest byte[]
+     * @param destLength int
+     * @return int
      */
     public static int copyBytes(long sourceOffset, byte[] source,
                                 int sourceOff, int sourceLength,
@@ -1033,6 +1185,11 @@ public class ArrayUtil {
     /**
      * Copy the source to dest, returning dest or an enlarged array of result is
      * larger than dest.
+     *
+     * @param source byte[]
+     * @param dest byte[]
+     * @param destOffset int
+     * @return byte[]
      */
     public static byte[] copyBytes(byte[] source, byte[] dest,
                                    int destOffset) {
@@ -1052,6 +1209,10 @@ public class ArrayUtil {
 
     /**
      * Convenience wrapper for System.arraycopy().
+     *
+     * @param source Object
+     * @param dest Object
+     * @param count int
      */
     public static void copyArray(Object source, Object dest, int count) {
         System.arraycopy(source, 0, dest, 0, count);
@@ -1089,6 +1250,11 @@ public class ArrayUtil {
 
     /**
      * Returns a range of elements of source from start to end of the array.
+     *
+     * @param source int[]
+     * @param start int
+     * @param count int
+     * @return int[]
      */
     public static int[] arraySlice(int[] source, int start, int count) {
 
@@ -1101,6 +1267,10 @@ public class ArrayUtil {
 
     /**
      * Fills part of the array with a value.
+     *
+     * @param array char[]
+     * @param offset int
+     * @param value char
      */
     public static void fillArray(char[] array, int offset, char value) {
 
@@ -1113,6 +1283,10 @@ public class ArrayUtil {
 
     /**
      * Fills part of the array with a value.
+     *
+     * @param array byte[]
+     * @param offset int
+     * @param value byte
      */
     public static void fillArray(byte[] array, int offset, byte value) {
 
@@ -1125,6 +1299,9 @@ public class ArrayUtil {
 
     /**
      * Fills the array with a value.
+     *
+     * @param array Object[]
+     * @param value Object
      */
     public static void fillArray(Object[] array, Object value) {
 
@@ -1137,6 +1314,9 @@ public class ArrayUtil {
 
     /**
      * Fills the int array with a value
+     *
+     * @param array int[]
+     * @param value int
      */
     public static void fillArray(int[] array, int value) {
 
@@ -1149,6 +1329,9 @@ public class ArrayUtil {
 
     /**
      * Fills the double array with a value
+     *
+     * @param array double[]
+     * @param value double
      */
     public static void fillArray(double[] array, double value) {
 
@@ -1161,6 +1344,9 @@ public class ArrayUtil {
 
     /**
      * Fills the int array with a value
+     *
+     * @param array boolean[]
+     * @param value boolean
      */
     public static void fillArray(boolean[] array, boolean value) {
 
@@ -1173,6 +1359,9 @@ public class ArrayUtil {
 
     /**
      * Returns a duplicates of an array.
+     *
+     * @param source Object
+     * @return Object
      */
     public static Object duplicateArray(Object source) {
 
@@ -1186,9 +1375,13 @@ public class ArrayUtil {
     }
 
     /**
-     * Returns the given array if newsize is the same as existing.
-     * Returns a new array of given size, containing as many elements of
-     * the original array as it can hold.
+     * Returns the given array if newsize is the same as existing. Returns a new
+     * array of given size, containing as many elements of the original array as
+     * it can hold.
+     *
+     * @param source Object
+     * @param newsize int
+     * @return Object
      */
     public static Object resizeArrayIfDifferent(Object source, int newsize) {
 
@@ -1211,9 +1404,13 @@ public class ArrayUtil {
     }
 
     /**
-     * Returns a new array of given size, containing as many elements of
-     * the original array as it can hold. N.B. Always returns a new array
-     * even if newsize parameter is the same as the old size.
+     * Returns a new array of given size, containing as many elements of the
+     * original array as it can hold. N.B. Always returns a new array even if
+     * newsize parameter is the same as the old size.
+     *
+     * @param source Object
+     * @param newsize int
+     * @return Object
      */
     public static Object resizeArray(Object source, int newsize) {
 
@@ -1230,12 +1427,37 @@ public class ArrayUtil {
         return newarray;
     }
 
+    /**
+     * Returns a new array containing the elements of parameter source with an
+     * added element at the end. Parameter addition is an Object to add.
+     *
+     * @param <T> type of array element
+     * @param source T[]
+     * @param addition element to append to array
+     * @return T[]
+     */
+    public static <T> T[] toAdjustedArray(T[] source, T addition) {
+
+        int size     = source.length;
+        T[] newArray = Arrays.copyOf(source, size + 1);
+
+        newArray[size] = addition;
+
+        return newArray;
+    }
+
     /**
      * Returns an array containing the elements of parameter source, with one
      * element removed or added. Parameter adjust {-1, +1} indicates the
      * operation. Parameter colindex indicates the position at which an element
-     * is removed or added. Parameter addition is an Object to add when
-     * adjust is +1.
+     * is removed or added. Parameter addition is an Object to add when adjust
+     * is +1.
+     *
+     * @param source Object
+     * @param addition Object
+     * @param colindex int
+     * @param adjust int
+     * @return Object
      */
     public static Object toAdjustedArray(Object source, Object addition,
                                          int colindex, int adjust) {
@@ -1251,13 +1473,18 @@ public class ArrayUtil {
 
     /**
      * Copies elements of source to dest. If adjust is -1 the element at
-     * colindex is not copied. If adjust is +1 that element is filled with
-     * the Object addition. All the rest of the elements in source are
-     * shifted left or right accordingly when they are copied. If adjust is 0
-     * the addition is copied over the element at colindex.
+     * colindex is not copied. If adjust is +1 that element is filled with the
+     * Object addition. All the rest of the elements in source are shifted left
+     * or right accordingly when they are copied. If adjust is 0 the addition is
+     * copied over the element at colindex. No checks are performed on array
+     * sizes and an exception is thrown if they are not consistent with the
+     * other arguments.
      *
-     * No checks are performed on array sizes and an exception is thrown
-     * if they are not consistent with the other arguments.
+     * @param source Object
+     * @param dest Object
+     * @param addition Object
+     * @param colindex int
+     * @param adjust int
      */
     public static void copyAdjustArray(Object source, Object dest,
                                        Object addition, int colindex,
@@ -1304,10 +1531,14 @@ public class ArrayUtil {
     /**
      * Similar to single slot adjusted copy, with multiple slots added or
      * removed. The colindex array is the ordered lists the slots to be added or
-     * removed. The adjust argument can be {-1, +1) for remove or add.
+     * removed. The adjust argument can be {-1, +1) for remove or add. No checks
+     * are performed on array sizes and no exception is thrown if they are not
+     * consistent with the other arguments.
      *
-     * No checks are performed on array sizes and no exception is thrown
-     * if they are not consistent with the other arguments.
+     * @param source Object[]
+     * @param dest Object[]
+     * @param colindex int[]
+     * @param adjust int
      */
     public static void copyAdjustArray(Object[] source, Object[] dest,
                                        int[] colindex, int adjust) {
@@ -1320,10 +1551,9 @@ public class ArrayUtil {
 
         for (int i = 0, j = 0, counter = 0;
                 i < source.length && j < dest.length; i++, j++) {
-            int adjustPos = -1;
 
             if (counter < colindex.length) {
-                adjustPos = colindex[counter];
+                int adjustPos = colindex[counter];
 
                 if (adjust > 0) {
                     if (adjustPos == j) {
@@ -1411,6 +1641,11 @@ public class ArrayUtil {
     /**
      * similar to the function with single colindex, but with multiple
      * adjustments.
+     *
+     * @param colarr int[]
+     * @param colindex int[]
+     * @param adjust int
+     * @return int[]
      */
     public static int[] toAdjustedColumnArray(int[] colarr, int[] colindex,
             int adjust) {
@@ -1420,7 +1655,6 @@ public class ArrayUtil {
         }
 
         int[] intarr = new int[colarr.length];
-        int   j      = 0;
 
         if (adjust == 0) {
             for (int i = 0; i < colarr.length; i++) {
@@ -1585,6 +1819,10 @@ public class ArrayUtil {
 
     /**
      * Returns true if char argument is in array.
+     *
+     * @param ch char
+     * @param array char[]
+     * @return boolean
      */
     public static boolean isInSortedArray(char ch, char[] array) {
 
@@ -1595,10 +1833,9 @@ public class ArrayUtil {
 
         int low  = 0;
         int high = array.length;
-        int mid  = 0;
 
         while (low < high) {
-            mid = (low + high) >>> 1;
+            int mid = (low + high) >>> 1;
 
             if (ch < array[mid]) {
                 high = mid;
@@ -1644,7 +1881,6 @@ public class ArrayUtil {
      */
     public static boolean containsAny(Object[] arra, Object[] arrb) {
 
-        mainLoop:
         for (int i = 0; i < arrb.length; i++) {
             for (int j = 0; j < arra.length; j++) {
                 if (arrb[i] == arra[j] || arrb[i].equals(arra[j])) {
@@ -1757,8 +1993,12 @@ public class ArrayUtil {
     }
 
     /**
-     * Compares two arrays. Returns -1, 0, +1. If one array is shorter and
-     * all the elements are equal to the other's elements, -1 is returned.
+     * Compares two arrays. Returns -1, 0, +1. If one array is shorter and all
+     * the elements are equal to the other's elements, -1 is returned.
+     *
+     * @param a byte[]
+     * @param b byte[]
+     * @return int
      */
     public static int compare(byte[] a, byte[] b) {
         return compare(a, 0, a.length, b, 0, b.length);
@@ -1792,7 +2032,12 @@ public class ArrayUtil {
     }
 
     /**
-     * uses 2**scale form and returns a multiple of unit that is larger or equal to value
+     * uses 2**scale form and returns a multiple of unit that is larger or equal
+     * to value
+     *
+     * @param value long
+     * @param unit long
+     * @return long
      */
     public static long getBinaryMultipleCeiling(long value, long unit) {
 
@@ -1806,7 +2051,12 @@ public class ArrayUtil {
     }
 
     /**
-     * uses 2**scale form and returns a multiple of this that is larger or equal to value
+     * uses 2**scale form and returns a multiple of this that is larger or equal
+     * to value
+     *
+     * @param value long
+     * @param scale int
+     * @return long
      */
     public static long getBinaryNormalisedCeiling(long value, int scale) {
 
@@ -1821,7 +2071,11 @@ public class ArrayUtil {
     }
 
     /**
-     * returns the largest value that is a power of 2 and larger or equal to value
+     * returns the smallest value that is a power of 2 and larger or equal to
+     * value
+     *
+     * @param value long
+     * @return long
      */
     public static long getBinaryNormalisedCeiling(long value) {
 
@@ -1836,6 +2090,10 @@ public class ArrayUtil {
 
     /**
      * returns true if log2 n is in the range (0, max)
+     *
+     * @param n int
+     * @param max int
+     * @return boolean
      */
     public static boolean isTwoPower(int n, int max) {
 
@@ -1851,7 +2109,11 @@ public class ArrayUtil {
     }
 
     /**
-     * returns the largest value that is 0 or a power of 2 and is smaller or equal to n
+     * returns the largest value that is 0 or a power of 2 and is smaller or
+     * equal to n
+     *
+     * @param n int
+     * @return int
      */
     public static int getTwoPowerFloor(int n) {
 
@@ -1865,7 +2127,11 @@ public class ArrayUtil {
     }
 
     /**
-     * returns the log2 of largest value that is 0 or a power of 2 and is smaller or equal to n
+     * returns the log2 of largest value that is 0 or a power of 2 and is
+     * smaller or equal to n
+     *
+     * @param n int
+     * @return int
      */
     public static int getTwoPowerScale(int n) {
 
@@ -1887,8 +2153,12 @@ public class ArrayUtil {
     }
 
     /**
-     * a and b must be both positive
-     * returns (a / b) or (a / b) + 1 if remainder is larger than zero
+     * a and b must be both positive returns (a / b) or (a / b) + 1 if remainder
+     * is larger than zero
+     *
+     * @param a int
+     * @param b int
+     * @return int
      */
     public static int cdiv(int a, int b) {
 
