@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2020, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -195,13 +195,13 @@ implements DataInput {
         return Double.longBitsToDouble(readLong());
     }
 
-    public int skipBytes(int n) throws IOException {
+    public int skipBytes(int n) {
         return (int) skip(n);
     }
 
-    public String readLine() throws IOException {
+    public String readLine() {
 
-        /** @todo: this will probably be useful */
+        /* @todo: this will probably be useful */
         throw new java.lang.UnsupportedOperationException("not implemented");
     }
 
@@ -278,5 +278,5 @@ implements DataInput {
         pos = mark;
     }
 
-    public void close() throws IOException {}
+    public void close() {}
 }
