@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,8 @@
 
 package org.hsqldb.map;
 
+import java.util.Arrays;
+
 import org.hsqldb.lib.ArrayUtil;
 
 /**
@@ -109,9 +111,7 @@ public class BitMap {
      */
     public void reset() {
 
-        for (int i = 0; i < map.length; i++) {
-            map[i] = 0;
-        }
+        Arrays.fill(map, 0);
 
         limitPos = initialSize;
     }
