@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2020, The HSQL Development Group
+/* Copyright (c) 2001-2021, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,6 @@ import org.hsqldb.NumberSequence;
 import org.hsqldb.Row;
 import org.hsqldb.Session;
 import org.hsqldb.SessionInterface;
-import org.hsqldb.SqlInvariants;
 import org.hsqldb.Statement;
 import org.hsqldb.Table;
 import org.hsqldb.TableBase;
@@ -86,7 +85,7 @@ import org.hsqldb.types.Type;
  *  storage.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.1
+ * @version 2.6.0
  * @since 1.7.0
  */
 public class Logger implements EventLogInterface {
@@ -192,7 +191,7 @@ public class Logger implements EventLogInterface {
      *  If the specified Database object is a new database, its database
      *  files are first created.
      *
-     * @throws  HsqlException if there is a problem, such as the case when
+     * @throws  org.hsqldb.HsqlException if there is a problem, such as the case when
      *      the specified files are in use by another process
      */
     public void open() {
@@ -337,7 +336,7 @@ public class Logger implements EventLogInterface {
         logsStatements = true;
         loggingEnabled = propLogData && !database.isFilesReadOnly();
 
-        if (database.getNameString().length() == 0) {
+        if (database.getNameString().isEmpty()) {
             database.setDatabaseName(newUniqueName());
         }
 
@@ -735,7 +734,7 @@ public class Logger implements EventLogInterface {
 
         String name = database.getNameString();
 
-        if (name.length() == 0) {
+        if (name.isEmpty()) {
 
             // The database unique name is set up at different times
             // depending on upgraded / exiting / new databases.
@@ -980,7 +979,7 @@ public class Logger implements EventLogInterface {
      *  files, in order to ensure as much as possible the ACID properties
      *  of the database.
      *
-     * @throws  HsqlException if there is a problem checkpointing the
+     * @throws  org.hsqldb.HsqlException if there is a problem checkpointing the
      *      database
      */
     public void checkpoint(Session session, boolean defrag, boolean lobs) {
@@ -996,7 +995,13 @@ public class Logger implements EventLogInterface {
                 checkpointInternal(session, defrag);
 
                 if (lobs) {
-                    database.lobManager.deleteUnusedLobs();
+                    Result result = database.lobManager.deleteUnusedLobs();
+
+                    if (log != null && result.getUpdateCount() > 0) {
+                        log.synchLog();
+                        logDetailEvent("Deleted unused LOBs, count: "
+                                       + result.getUpdateCount());
+                    }
                 }
             }
         } finally {
@@ -2254,8 +2259,8 @@ public class Logger implements EventLogInterface {
 
             if (compressed != nameImpliesCompress) {
                 throw Error.error(null, ErrorCode.COMPRESSION_SUFFIX_MISMATCH,
-                                  0, new Object[] {
-                    Boolean.valueOf(compressed), archiveFile.getName()
+                                  0, new String[] {
+                    String.valueOf(compressed), archiveFile.getName()
                 });
             }
 
